# Keyword: `constexpr`

## Problem in Previous Versions (C++98/C++03):
In earlier versions of C++, constant expressions were limited to simple constant values, such as literals, or values that could be computed at compile time using preprocessor macros or `const` variables. However, `const` variables did not guarantee that their values would be computed at compile time. This limitation could lead to less efficient code, as some values that could theoretically be computed at compile time were instead calculated at runtime.

## Example 1: c99
In the code below, the `square(5)` function might not be evaluated at compile time, even though value is declared as `const`. This can lead to potential inefficiencies and unintended runtime behavior, such as errors when using value in contexts where a compile-time constant is required (e.g., as an array size).
```cpp
#include <iostream>

int square(int x) {
    return x * x;
}

const int value = square(5);  // Not guaranteed to be a compile-time constant

int main() {
    int arr[value];  // Might cause a runtime error if `value` is not a constant
    std::cout << "Value: " << value << std::endl;
    return 0;
}
```

## Example 2: c11
C++11 introduced `constexpr`, which allows you to define functions and variables that are guaranteed to be evaluated at compile time, provided that all inputs are known at compile time. This makes code more efficient and allows for safer and more predictable behavior.
```cpp
#include <iostream>

constexpr int square(int x) {
    return x * x;
}

constexpr int value = square(5);  // Guaranteed to be a compile-time constant

int main() {
    int arr[value];  // Safe to use `value` as an array size
    std::cout << "Value: " << value << std::endl;
    return 0;
}
```

### Explanation:
In this updated example, the `square` function is declared as `constexpr`, which guarantees that it will be evaluated at compile time if given constant arguments. The variable value is also declared as `constexpr`, ensuring that it is a compile-time constant and can be safely used in contexts that require constant expressions, such as the size of an array.

## Key Takeaway:
The introduction of `constexpr` in C++11 solves the problem of ensuring that certain expressions are evaluated at compile time. This leads to more efficient code, as the computations are performed during compilation rather than at runtime. It also enhances the safety and predictability of code, especially when working with constants that need to be known at compile time.
